# -*- coding: utf-8 -*-
from z3c.form.interfaces import IEditForm, IAddForm

from zope import schema
from zope.interface import alsoProvides

from plone.app.textfield import RichText as RichTextField
from plone.app.z3cform.widget import RichTextFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider

from plone.supermodel import model

from collective_folderprotection import _


class IPasswordProtected(model.Schema):
    """Behavior interface to enable password protection"""

    passw_hash = schema.Password(
        title=_("Password"),
        description=_(
            "Choose a password to protect this object and, if it is a "
            "folder, its children."
        ),
        required=False,
    )

    passw_reason = RichTextField(
        title=_("Reason"),
        description=_(
            "You can add a reason on why this folder is being protected "
            "by a password."
        ),
        required=False,
    )
    form.widget("passw_reason", RichTextFieldWidget)

    form.omitted("passw_hash")
    form.omitted("passw_reason")


alsoProvides(IPasswordProtected, IFormFieldProvider)


class IDeleteProtected(model.Schema):
    """Marker interface to enable delete protection behavior"""

    delete_protection = schema.Bool(
        title=_("Delete protection"),
        description=_(
            "Mark this checkbox if you want to protect this object, and its "
            "children (if this is a folder) from being deleted. NOTE: In "
            "folders, the delete protection will only protect direct "
            "children, it will not recurse into subfolders."
        ),
        required=False,
    )

    form.omitted("delete_protection")
    form.no_omit(IEditForm, "delete_protection")
    form.no_omit(IAddForm, "delete_protection")


alsoProvides(IDeleteProtected, IFormFieldProvider)


class IRenameProtected(model.Schema):
    """Marker interface to enable rename protection behavior"""

    rename_protection = schema.Bool(
        title=_("Rename protection"),
        description=_(
            "Mark this checkbox if you want to protect this object, and its "
            "children (if this is a folder) from being renamed. NOTE: In "
            "folders, the rename protection will only protect direct "
            "children, it will not recurse into subfolders."
        ),
        required=False,
    )

    form.omitted("rename_protection")
    form.no_omit(IEditForm, "rename_protection")
    form.no_omit(IAddForm, "rename_protection")


alsoProvides(IRenameProtected, IFormFieldProvider)



# -*- coding: utf-8 -*-


class RenameProtected(object):
    def __init__(self, context):
        self.context = context

    def _get_rename_protection(self):
        result = False
        if self.context.rename_protection:
            result = True
        return result

    def _set_rename_protection(self, value):
        self.context.rename_protection = value

    rename_protection = property(_get_rename_protection, _set_rename_protection)



# -*- coding: utf-8 -*-
import unittest

from collective_folderprotection.exceptions import RenameProtectionException
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

from collective_folderprotection.behaviors.interfaces import IRenameProtected
from collective_folderprotection.testing import INTEGRATION_TESTING


class TestRenameProtect(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # Create a folderish protected
        self.portal.invokeFactory("folderish_protected", "protected")
        self.protected = self.portal["protected"]
        adapter = IRenameProtected(self.protected)
        adapter.rename_protection = True
        self.protected.invokeFactory("Document", "internal")

        # Create another folderish protected, with protection disabled
        self.portal.invokeFactory("folderish_protected", "protected-disabled")
        self.protected_disabled = self.portal["protected-disabled"]
        adapter = IRenameProtected(self.protected_disabled)
        adapter.rename_protection = False
        self.protected_disabled.invokeFactory("Document", "internal")

        # And a folderish unprotected
        self.portal.invokeFactory("folderish_not_protected", "not-protected")
        self.not_protected = self.portal["not-protected"]
        self.not_protected.invokeFactory("Document", "internal")

    def test_unable_to_rename_protected(self):
        self.assertRaises(
            RenameProtectionException,
            self.protected.manage_renameObjects,
            ["internal"],
            ["new-internal"],
        )
        self.assertIn("internal", self.protected)
        self.assertNotIn("new-internal", self.protected)
        self.assertRaises(
            RenameProtectionException,
            self.portal.manage_renameObjects,
            ["protected"],
            ["new-protected"],
        )
        self.assertIn("protected", self.portal)
        self.assertNotIn("new-protected", self.portal)

    def test_able_to_rename_unprotected(self):
        self.not_protected.manage_renameObjects(["internal"], ["new-internal"])
        self.assertNotIn("internal", self.not_protected)
        self.assertIn("new-internal", self.not_protected)
        self.portal.manage_renameObjects(["not-protected"], ["new-not-protected"])
        self.assertNotIn("not-protected", self.portal)
        self.assertIn("new-not-protected", self.portal)

    def test_able_to_rename_disabled(self):
        self.protected_disabled.manage_renameObjects(["internal"], ["new-internal"])
        self.assertNotIn("internal", self.protected_disabled)
        self.assertIn("new-internal", self.protected_disabled)
        self.portal.manage_renameObjects(
            ["protected-disabled"], ["new-protected-disabled"]
        )
        self.assertNotIn("protected-disabled", self.portal)
        self.assertIn("new-protected-disabled", self.portal)



# -*- coding: utf-8 -*-
from datetime import timedelta

PROJECTNAME = "collective_folderprotection"

HASHES_ANNOTATION_KEY = "collective_folderprotection_valid_hashes"
HASH_COOKIE_KEY = "collective_folderprotection_hash"

# How much time the password should be remembered.
# XXX: Should this be configurable ?
TIME_TO_LIVE = timedelta(days=2)



# -*- extra stuff goes here -*-
from zope.i18nmessageid import MessageFactory

_ = MessageFactory("collective_folderprotection")


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# -*- coding: utf-8 -*-
import unittest

from collective_folderprotection.exceptions import DeleteProtectionException
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

from collective_folderprotection.behaviors.interfaces import IDeleteProtected
from collective_folderprotection.testing import INTEGRATION_TESTING


class TestDelProtect(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # Create a folderish protected
        self.portal.invokeFactory("folderish_protected", "protected")
        self.protected = self.portal["protected"]
        adapter = IDeleteProtected(self.protected)
        adapter.delete_protection = True
        self.protected.invokeFactory("Document", "internal")

        # Create another folderish protected, with protection disabled
        self.portal.invokeFactory("folderish_protected", "protected-disabled")
        self.protected_disabled = self.portal["protected-disabled"]
        adapter = IDeleteProtected(self.protected_disabled)
        adapter.delete_protection = False
        self.protected_disabled.invokeFactory("Document", "internal")

        # And a folderish unprotected
        self.portal.invokeFactory("folderish_not_protected", "not-protected")
        self.not_protected = self.portal["not-protected"]
        self.not_protected.invokeFactory("Document", "internal")

    def test_unable_to_remove_protected(self):
        self.assertRaises(DeleteProtectionException, self.protected.manage_delObjects, "internal")
        self.assertIn("internal", self.protected)
        self.assertRaises(DeleteProtectionException, self.portal.manage_delObjects, "protected")
        self.assertIn("protected", self.portal)

    def test_able_to_remove_unprotected(self):
        self.not_protected.manage_delObjects("internal")
        self.assertNotIn("internal", self.not_protected)
        self.portal.manage_delObjects("not-protected")
        self.assertNotIn("not-protected", self.portal)

    def test_able_to_remove_disabled(self):
        self.protected_disabled.manage_delObjects("internal")
        self.assertNotIn("internal", self.protected_disabled)
        self.portal.manage_delObjects("protected-disabled")
        self.assertNotIn("protected-disabled", self.portal)



# -*- coding: utf-8 -*-
import unittest

from plone.browserlayer.utils import registered_layers

from collective_folderprotection.testing import INTEGRATION_TESTING


class TestSetup(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

    def test_addon_layer(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertTrue(
            "IFolderProtectLayer" in layers, "add-on layer was not installed"
        )



# -*- coding: utf-8 -*-
from Acquisition import aq_parent
from collective_folderprotection.behaviors.interfaces import IPasswordProtected
from plone.app.layout.viewlets.common import ViewletBase
from plone.app.contenttypes.interfaces import IFolder
from plone.app.textfield.interfaces import IRichTextValue
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot


class ReasonViewlet(ViewletBase):
    """Displays alert messages with the reason on why this content is pw protected"""

    context_pw_protected = False
    reason = ""

    def update(self):
        super(ReasonViewlet, self).update()
        if self.request.response.getStatus() == 401:
            return
        context = self.context
        if not IPloneSiteRoot.providedBy(context):
            if not IFolder.providedBy(self.context):
                context = aq_parent(self.context)

        passw_behavior = IPasswordProtected(context, None)
        if passw_behavior:
            self.context_pw_protected = passw_behavior.is_password_protected()

            if self.context_pw_protected:
                reason = passw_behavior.passw_reason
                if reason:
                    if IRichTextValue.providedBy(reason):
                        self.reason = reason.output_relative_to(context)
                    else:
                        self.reason = reason
